@@ -159,7 +159,7 @@ def my_ip(args: MyoriginArgs) -> str:
         filename=args.logfile if args.logfile != '-' else None,
         filemode='a',
     )
-    logger = logging.getLogger(__name__)
+    logger = logging.getLogger('myorigin')
     log_levels = [logging.CRITICAL, logging.ERROR, logging.WARNING, logging.INFO, logging.DEBUG]
     try:
         logger.setLevel(log_levels[args.log_level])
@@ -201,8 +201,9 @@ def http_timer():  # configure signals to record beginning and end of http reque
 
 async def http_get(url: str, session: aiohttp.ClientSession, max_size=-1, ip_version=0):
     timer_marks = list()
-    # change user agent string for better site compatibility
-    # via: wget -O- -q http://httpbin.org/get |grep User-Agent
+    # change user agent string for better site compatibility and anonymity, though one site
+    # ... suggests a Firefox user agent (https://www.whatismyip.com/automation-rules/);
+    # Wget agent retrieved via: wget -O- -q http://httpbin.org/get |grep User-Agent
     headers = {'User-Agent': 'Wget/1.20.3 (linux-gnu)'}
     # headers = {'User-Agent': 'Python/3.8 aiohttp/3.8.1'}  # default on my system
     try:
@@ -303,12 +304,10 @@ async def main_loop(args: MyoriginArgs, logger: logging.Logger) -> str:
     engine = init_db(args.dbfile)
     with Session(engine) as db_session:
         scores = dict()
-        if args.ip_version == 0:
-            statement = select(Parrot)
-        else:  # if user wants IPv4 only, ignore IPv6 DB rows; vice-versa
-            statement = select(Parrot).where(Parrot.ip_version == args.ip_version)
-        all_parrots = db_session.exec(statement)
+        all_parrots = db_session.exec(select(Parrot))
         for p in all_parrots:
+            if args.ip_version != 0 and args.ip_version != p.ip_version():  # only ones user wants
+                continue
             score = p.score()
             if score <= 0:
                 continue
@@ -340,7 +339,6 @@ async def main_loop(args: MyoriginArgs, logger: logging.Logger) -> str:
                     max_ipv0_count = 0  # max of IPv4 and IPv6
                     for v in (4, 6):
                         max_ip_count = 0 if len(ip_counts[v]) == 0 else max(ip_counts[v].values())
-                        # FIXME: for multiple IPs, maybe ensure most_common >= second_most_common + minimum_match more
                         if len(ip_counts[v]) > 1:
                             error = f"multiple IPs received: {ip_counts[v]}"
                             raise DoneWithJobs
@@ -354,34 +352,34 @@ async def main_loop(args: MyoriginArgs, logger: logging.Logger) -> str:
                     # spawn another get_ip() job if needed
                     wanted_count = args.minimum_match + args.overkill
                     jobs_count = max_ipv0_count + pending_jobs_count
-                    logger.debug(
-                        f"have {max_ipv0_count} IPs plus {pending_jobs_count} pending,"
-                        + f" want {wanted_count}, need {args.minimum_match}"
-                    )
+                    if logger.isEnabledFor(logging.DEBUG):
+                        try:
+                            last_debug_msg = main_loop.last_debug_msg
+                        except AttributeError:
+                            last_debug_msg = ""
+                        msg = f"have {max_ipv0_count} IPs plus {pending_jobs_count} pending,"
+                        msg += f" want {wanted_count}, need {args.minimum_match}"
+                        if msg != last_debug_msg:  # avoid repeating the same message
+                            logger.debug(msg)
+                        main_loop.last_debug_msg = msg
                     if args.overkill > 0 and wanted_count > jobs_count and len(scores) == 0:
-                        msg = f"not enough parrots for {wanted_count} requests"
+                        msg = f"not enough providers for {wanted_count} requests"
                         logger.warning(f"{msg}; ignoring '--overkill'")
                         args.overkill = 0
                         wanted_count = args.minimum_match + args.overkill
                     if wanted_count > jobs_count and pending_jobs_count < args.max_connections:
                         # TCPConnector(limit=...) does this too, but checking args.max_connections
-                        # ... here delays "not enough parrots" so we can collect more responses
+                        # ... here delays "not enough providers" so we can collect more responses
                         if len(scores) == 0:
-                            error = f"not enough parrots for {args.minimum_match} matches"
+                            error = f"not enough providers for {args.minimum_match} matches"
                             raise DoneWithJobs
                         p_id = weighted_random(scores)
                         statement = select(Parrot).where(Parrot.id == p_id)
                         p = db_session.exec(statement).one_or_none()
-                        assert p is not None
-                        a = p_id - (p_id % 100)  # 100 IDs per flock_data line
-                        b = a + 100  # top of range for a sinle flock_data line
-                        ids_to_del = [k for k in scores.keys() if a <= k < b]
-                        logger.debug(f"chose {p_id}, removing {ids_to_del}")
-                        for k in ids_to_del:  # eliminate all IDs from same flock_data line, e.g.
-                            del scores[k]  # don't check both http://ident.me and https://ident.me
-                        logger.debug(f"launching task IPv{p.ip_version} {p.url()}")
                         asyncio.create_task(get_ip(p, aio_session, q, ip_version=args.ip_version))
                         pending_jobs_count += 1
+                        logger.debug(f"launching task {p.id}")
+                        p.del_keys_of_same_parrot(scores, logger)  # don't use this parrot again
                     await asyncio.sleep(0.0001)
                     # process completed jobs
                     try:
@@ -421,15 +419,21 @@ async def main_loop(args: MyoriginArgs, logger: logging.Logger) -> str:
     return result
 
 
+def mkdir_r(path):  # like Linux `mkdir --parents`
+    if path == '':
+        return
+    base = os.path.dirname(path)
+    if not os.path.exists(base):
+        mkdir_r(base)
+    os.makedirs(path, exist_ok=True)
+    # except (PermissionError, FileNotFoundError, NotADirectoryError):
+    #     one of these will be raised if the directory cannot be created
+
+
 def db_pathname(create_dir=False):
     config_dir = platformdirs.user_config_dir(app_name())
     if create_dir:
-        try:
-            os.mkdir(config_dir)
-        except FileExistsError:
-            pass
-        # except (PermissionError, FileNotFoundError):
-        #     one of these will be raised if the directory cannot be created
+        mkdir_r(config_dir)
     return os.path.join(config_dir, f'data.sqlite')
 
 
